# frozen_string_literal: true

require 'json'
require 'httparty'
require 'api_signature'

module Plagiarism
  module Client
    module SignedRequest
      protected

      HEADER_KEYS = ApiSignature::Request::HEADER_KEYS

      %w[get post put delete].each do |kind|
        define_method kind do |path, **xargs|
          headers = xargs.fetch(:headers, {})
          headers = add_signature(headers, kind, path)
          xargs[:headers] = headers
          fullpath = File.join(Plagiarism::Client.endpoint, path)

          response = HTTParty.send(kind, fullpath, **xargs)
          JSON.parse(response.body)
        end
      end

      def add_signature(headers, request_method, path)
        @prefix ||= Plagiarism::Client.endpoint.gsub(/.*?(\/api\/v\d+.*)/, '\1')
        path = File.join(@prefix, path)
        options = signature_options(headers, request_method, path)

        HEADER_KEYS.each do |kind, header|
          headers[normalize_key(header)] = options[kind].to_s
        end

        headers
      end

      def signature_options(headers, request_method, path)
        options = {
          path: path,
          request_method: request_method.to_s.upcase,
          access_key: Plagiarism::Client.token,
          timestamp: timestamp_for_options(headers)
        }.with_indifferent_access

        options[:signature] = generate_signature(options)

        options
      end

      def timestamp_for_options(headers)
        header_key = HEADER_KEYS[:timestamp].gsub(/^HTTP_/, '')

        return headers[header_key] if headers.key? header_key
        Time.zone.now.to_i
      end

      def generate_signature(options)
        ApiSignature::Generator
          .new(options)
          .generate_signature(Plagiarism::Client.secret)
      end

      private

      def normalize_key(key)
        key.gsub(/^HTTP_/i, '').tr('_', '-').downcase
      end
    end
  end
end
